@@ -7,7 +7,7 @@ from typing import Callable
 
 from wcwidth import wcswidth, wcwidth
 
-from mizue.printer import Printer, TerminalColors
+from mizue.printer import Colorizer
 from mizue.util import Utility
 from .alignment import Alignment
 from .border_character_codes import BorderCharacterCodes
@@ -39,11 +39,15 @@ class Grid:
 
     def fill_screen(self):
         terminal_width = Utility.get_terminal_width()
-        total_width = sum([column.width for column in self.columns]) + (4 * len(self.columns))
-        if total_width < terminal_width:
-            remaining_width = terminal_width - sum([column.width for column in self.columns]) - (4 * len(self.columns))
+        used_width = self._get_grid_width()
+        print(used_width, terminal_width)
+        unused_width = terminal_width - used_width
+
+        # split the unused width between the columns
+        if unused_width > 0:
             for column in self.columns:
-                column.width += int(remaining_width / len(self.columns))
+                percentage = (column.width + 3) / used_width
+                column.width += int(unused_width * percentage)
 
     def print(self) -> None:
         """Print the grid"""
@@ -52,18 +56,18 @@ class Grid:
     def _buffer(self) -> str:
         buffer = [self._create_row_border(RowBorderPosition.TOP), os.linesep]
         title_list = [column.title for column in self.columns]
-        buffer.append(self._create_row(title_list, True, False)[0])
+        buffer.append(self._create_row(title_list, True, False, True)[0])
         buffer.append(os.linesep)
         buffer.append(self._create_row_border(RowBorderPosition.MIDDLE))
         buffer.append(os.linesep)
         for row_index, row in enumerate(self.data):
-            rendered_row, rendered_cells = self._create_row(row, False, True)
+            rendered_row, rendered_cells = self._create_row(row, False, True, True)
             for index, cell in enumerate(rendered_cells):
                 row[index] = cell
             row_with_offset_rows = Grid._get_multiline_cell_offset_rows(row)
             for offset_row in row_with_offset_rows:
                 offset_row = [Grid._replace_tabs(str(cell)) for cell in offset_row]
-                buffer.append(self._create_row(offset_row, False, False)[0])
+                buffer.append(self._create_row(offset_row, False, False, False)[0])
                 buffer.append(os.linesep)
             if self.separated_rows and row_index != len(self.data) - 1:
                 buffer.append(self._create_row_border(RowBorderPosition.MIDDLE))
@@ -71,7 +75,7 @@ class Grid:
         buffer.append(self._create_row_border(RowBorderPosition.BOTTOM))
         return "".join(buffer)
 
-    def _create_row(self, row: list[str], is_header_row: bool, wrap: bool) -> (str, list[str]):
+    def _create_row(self, row: list[str], is_header_row: bool, wrap: bool, use_renderer: bool) -> (str, list[str]):
         border_style = self._get_border_style()
         row_buffer = []
         rendered_cells: list[str] = []
@@ -82,25 +86,29 @@ class Grid:
             wide_char_count = sum([1 for char in cell if Grid._is_wide_char(char)])
             renderer = self._get_cell_renderer(column)
             width = column.width - wide_char_count
-            rendered_cell = renderer(CellRendererArgs(cell=cell, index=index, is_header=is_header_row,
-                                                      width=width))
-            wrapped_cell = Grid._split_string_keep_rgb_colors(rendered_cell, width) \
-                if wrap and column.wrap else [cell]
+
             colored_cells = []
-            if renderer is not None:
+            if renderer is not None and use_renderer:
+                rendered_cell = renderer(CellRendererArgs(cell=cell, index=index, is_header=is_header_row,
+                                                          width=width))
+                wrapped_cell = Grid._split_string_keep_rgb_colors(rendered_cell, width) \
+                    if wrap and column.wrap else [rendered_cell]
                 for wrapped_cell_part in wrapped_cell:
-                    rendered_cell_parts = renderer(CellRendererArgs(cell=wrapped_cell_part, index=index, is_header=is_header_row,
-                                                              width=column.width))
-                    rendered_cell_parts = self._format_cell_with_colors(rendered_cell_parts, column.width) \
-                        if not (wrap or column.wrap) else rendered_cell_parts
+                    # rendered_cell_parts = renderer(CellRendererArgs(cell=wrapped_cell_part, index=index,
+                    #                                                 is_header=is_header_row, width=column.width))
+                    # print(f"{rendered_cell_parts} :: TEST")
+                    rendered_cell_parts = self._format_cell_with_colors(wrapped_cell_part, column.width) \
+                        if not (wrap or column.wrap) else wrapped_cell_part
                     colored_cells.append(rendered_cell_parts)
             else:
+                wrapped_cell = Grid._split_string_keep_rgb_colors(cell, width) \
+                    if wrap and column.wrap else [cell]
                 for wrapped_cell_part in wrapped_cell:
                     rendered_cell_parts = self._format_cell_with_colors(wrapped_cell_part, column.width) \
                         if not (wrap or column.wrap) else wrapped_cell_part
                     colored_cells.append(rendered_cell_parts)
 
-            border = Printer.format_hex(border_style.VERTICAL, self.border_color) \
+            border = Colorizer.colorize(border_style.VERTICAL, self.border_color) \
                 if self.border_color else border_style.VERTICAL
             if index == 0:
                 row_buffer.append(f"{border}")
@@ -138,19 +146,13 @@ class Grid:
             if index != len(self.columns) - 1:
                 dash_list.append(middle)
         dash_list.append(right)
-        return Printer.format_hex("".join(dash_list), self.border_color) if self.border_color else "".join(dash_list)
+        return Colorizer.colorize("".join(dash_list), self.border_color) if self.border_color else "".join(dash_list)
 
     def _find_max_cell_width(self, column: Column) -> int:
         max_width = len(column.title)
         for row in self.data:
             cell = str(row[column.index])
-            length = 0
-            for char in str(cell):
-                if Grid._is_wide_char(char):
-                    length += 2
-                else:
-                    length += 1
-            length = length + 1 if Grid._has_variation_selector(cell) else length
+            length = Grid._get_terminal_width_of_cell(cell)
             max_width = max(max_width, length)
         return max_width
 
@@ -165,8 +167,6 @@ class Grid:
                 text = color_part[1]
                 text_width = wcswidth(text)
                 if processed_width + text_width <= column_width:
-                    if processed_width + text_width == column_width:
-                        text = text[:-1] + "…" if Grid._is_wide_char(text[-1]) else text
                     formatted_text = f"{color_part[0]}{text}{color_part[2]}"
                     formatted_parts.append(formatted_text)
                     processed_width += text_width
@@ -185,13 +185,13 @@ class Grid:
             if col_width == 1:
                 if len(cell) == 1:
                     return cell[0] if wcwidth(cell[0]) == 1 else "…"
-                return "…" if wcwidth(cell[0]) == 1 else "…"
+                return "…" if len(cell) > 1 and wcwidth(cell[0]) == 1 else " "
             if col_width == 2:
                 if len(cell) == 1:
                     return cell[0] if wcwidth(cell[0]) == 1 else "…"
                 if len(cell) == 2:
                     return cell[0] + cell[1] if wcwidth(cell[0]) == 1 and wcwidth(cell[1]) == 1 else "…"
-                return "…" if wcwidth(cell[0]) == 1 else "…"
+                return "…" if len(cell) > 1 and wcwidth(cell[0]) == 1 else " "
             if col_width == 3:
                 if wcwidth(cell[0]) == 2:
                     return cell[0] + "…"
@@ -249,9 +249,12 @@ class Grid:
     @staticmethod
     def _get_default_cell_renderer(args: CellRendererArgs) -> str:
         if args.is_header:
-            return Printer.format_hex(args.cell, '#FFCC75')
+            return Colorizer.colorize(args.cell, '#FFCC75')
         return args.cell
 
+    def _get_grid_width(self) -> int:
+        return sum([column.width for column in self.columns]) + (3 * len(self.columns) + 1)
+
     @staticmethod
     def _get_left_cell_space(column: Column, cell: str) -> str:
         cell_terminal_width = Grid._get_terminal_width_of_cell(cell)
@@ -297,7 +300,8 @@ class Grid:
 
     @staticmethod
     def _get_terminal_width_of_cell(text):
-        return sum([2 if wcswidth(char) == 2 else 1 for char in text])
+        return (sum([2 if wcswidth(char) == 2 else 1 for char in text]) +
+                sum([1 for char in text if Grid._is_variation_selector(char)]))
 
     @staticmethod
     def _get_visible_text(text: str, max_width: int) -> str:
@@ -335,7 +339,7 @@ class Grid:
             column.width = column_setting["width"] if "width" in column_setting else self._find_max_cell_width(column)
             columns.append(column)
         self._columns = columns
-        # self._resize_columns_to_fit()
+        self._resize_columns_to_fit()
 
     @staticmethod
     def _replace_tabs(text: str) -> str:
@@ -346,14 +350,17 @@ class Grid:
         new_column_width = int(terminal_width / len(self.columns))
         long_columns = [column for column in self.columns if column.width >= new_column_width]
 
-        remaining_width = 0
+        unused_width = 0
         for column in self.columns:
             if column.width > new_column_width:
                 column.width = new_column_width
             else:
-                remaining_width += (new_column_width - column.width - 4 * len(self.columns))
+                unused_column_width = new_column_width - column.width
+                unused_width = unused_width + unused_column_width
 
-        padding = int(remaining_width / len(long_columns)) if len(long_columns) > 0 else 0
+        unused_width = unused_width - (3 * len(self.columns)) - 1
+
+        padding = int(unused_width / len(long_columns)) if len(long_columns) > 0 else 0
         for column in long_columns:
             column.width += padding
 
@@ -381,7 +388,7 @@ class Grid:
         line_group = 0
         diff = 0
         while dx < len(dict.keys()):
-            color, string, reset = dict[dx]
+            color, string, reset = dict.get(dx)
             line_width = wcswidth(line)
             string_width = wcswidth(string)
             if string_width + line_width == line_length:
@@ -418,11 +425,12 @@ class Grid:
             for color, string, reset, line_group in group:
                 line += color + string + reset
             lines.append(line)
+            # print(line)
         return lines
 
     @staticmethod
-    def _split_text_into_color_parts(text: str) -> list[tuple[str, str, str]]:
-        matcher = re.compile(r"(\x1b\[38;2;\d+;\d+;\d+m(?:\x1b\[\d+m)*)(.*?)(\x1b\[0m)")
+    def _split_text_into_color_parts(text: str, matcher_regex=None) -> list[tuple[str, str, str]]:
+        matcher = matcher_regex or re.compile(r"((?:\x1b\[.*?m)+)(.*?)(\x1b\[0m(?:\x1b\[.*?m)*)")
         substring_tuples = []
         for m in re.finditer(matcher, text):
             substring_tuples.append(m.span())
@@ -441,6 +449,9 @@ class Grid:
         for substring in substrings:
             if re.match(matcher, substring):
                 g = re.match(matcher, substring).groups()
+                if "\x1b" in g[1]:
+                    return Grid._split_text_into_color_parts(text, re.compile(
+                        r"(\x1b\[38;2;\d+;\d+;\d+m(?:\x1b\[\d+m)*)(.*?)(\x1b\[0m)"))
                 parts.append(g)
             else:
                 parts.append(("", substring, ""))
